//*****************************************************************************
//  Copyright (c) 2021 Glenfly Tech Co., Ltd.. 
//  All Rights Reserved.
//
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Glenfly Tech Co., Ltd..;
//  the contents of this file may not be disclosed to third parties, copied or
//  duplicated in any form, in whole or in part, without the prior written
//  permission of Glenfly Tech Co., Ltd..
//
//  The copyright of the source code is protected by the copyright laws of the People's
//  Republic of China and the related laws promulgated by the People's Republic of China
//  and the international covenant(s) ratified by the People's Republic of China. 
//*****************************************************************************

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/font.h>
#include <linux/mutex.h>
#include <linux/videodev2.h>
#include <linux/kthread.h>
#include <linux/freezer.h>
#include <linux/delay.h>

#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <asm/io.h>
#ifdef   __arm__
#include <asm/system.h>
#endif
//#include <asm/uaccess.h>

#include <media/videobuf2-vmalloc.h>
#include <media/v4l2-device.h>
#include <media/v4l2-ioctl.h>
#include <media/v4l2-ctrls.h>
#include <media/v4l2-fh.h>
#include <media/v4l2-event.h>
#include <media/v4l2-common.h>

#include "v4l2_vip_def.h"
#include "gf_vip_video.h"

#define  TRACE_INFO(item)   printk("KMD: %s ctrl handle.\n", #item)

int h8v_ctrl_set_flash(struct h8_video_dev* dev)
{
    TRACE_INFO(flash);
    return 0;
}
int h8v_ctrl_set_AF(struct h8_video_dev* dev)
{
    TRACE_INFO(AF);
    return 0;
}
int h8v_ctrl_set_AF_area(struct h8_video_dev* dev)
{
    TRACE_INFO(AF_area);
    return 0;
}
int h8v_ctrl_set_AWB(struct h8_video_dev* dev)
{
    TRACE_INFO(AWB);
    return 0;
}
int h8v_ctrl_set_AE(struct h8_video_dev* dev)
{
    TRACE_INFO(AE);
    return 0;
}
int h8v_ctrl_set_metering(struct h8_video_dev* dev)
{
    TRACE_INFO(metering);
    return 0;
}
int h8v_ctrl_set_metering_area(struct h8_video_dev* dev)
{
    TRACE_INFO(metering_area);
    return 0;
}
int h8v_ctrl_set_ISO(struct h8_video_dev* dev)
{
    TRACE_INFO(ISO);
    return 0;
}
int h8v_ctrl_set_sharpness(struct h8_video_dev* dev)
{
    TRACE_INFO(sharpness);
    return 0;
}
int h8v_ctrl_set_antibanding(struct h8_video_dev* dev)
{
    TRACE_INFO(antibanding);
    return 0;
}
int h8v_ctrl_set_rgb2yuv_mode(struct h8_video_dev* dev)
{
    TRACE_INFO(yuy2_mod);
    return 0;
}

int h8v_ctrl_set_rgb2yuv_matrix(struct h8_video_dev* dev)
{
    TRACE_INFO(yuy2_matrix);
    return 0;
}

int h8v_ctrl_set_tonemap(struct h8_video_dev* dev)
{
    TRACE_INFO(tonemap);
    return 0;
}
int h8v_ctrl_set_tonemap_curve(struct h8_video_dev* dev)
{
    TRACE_INFO(tonemap_curve);
    return 0;
}
int h8v_ctrl_set_special_effect(struct h8_video_dev* dev)
{
    TRACE_INFO(special_effect);
    return 0;
}
int h8v_ctrl_set_saturation(struct h8_video_dev* dev)
{
    TRACE_INFO(saturation);
    return 0;
}
int h8v_ctrl_set_contrast(struct h8_video_dev* dev)
{
    TRACE_INFO(contrast);
    return 0;
}
int h8v_ctrl_set_scene(struct h8_video_dev* dev)
{
    TRACE_INFO(scene);
    return 0;
}
int h8v_ctrl_set_scenario_mode(struct h8_video_dev* dev)
{
    TRACE_INFO(scenario);
    return 0;
}
int h8v_ctrl_set_shading_correct(struct h8_video_dev* dev)
{
    TRACE_INFO(shading_correct);
    return 0;
}

int h8v_ctrl_set_exposure_time(struct h8_video_dev* dev)
{
    TRACE_INFO(exposure_time);
    return 0;
}
int h8v_ctrl_set_manual_wb(struct h8_video_dev* dev)
{
    TRACE_INFO(manual_wb);
    return 0;
}

int h8v_ctrl_set_stream_init(struct h8_video_dev* dev)
{
    TRACE_INFO(stream_init);
    // should save max stream width_height, to avoid AP set override.
    // to do ...
    return 0;
}

int h8v_ctrl_set_main(struct h8_video_dev* dev);

h8v_ctrl_t h8v_ctrls[] =
{
    [V4L2_H8_CTRL_NR(V4L2_CID_FLASH_MODE)]                 = h8v_ctrl_set_flash,
    [V4L2_H8_CTRL_NR(V4L2_CID_AUTO_FOCUS_MODE)]            = h8v_ctrl_set_AF,
    [V4L2_H8_CTRL_NR(V4L2_CID_AUTO_FOCUS_AREA_SET)]        = h8v_ctrl_set_AF_area,
    [V4L2_H8_CTRL_NR(V4L2_CID_AWB_MODE)]                   = h8v_ctrl_set_AWB,
    [V4L2_H8_CTRL_NR(V4L2_CID_AUTO_EXPOSURE_SET)]          = h8v_ctrl_set_AE,

    [V4L2_H8_CTRL_NR(V4L2_CID_METERING_MODE)]              = h8v_ctrl_set_metering,
    [V4L2_H8_CTRL_NR(V4L2_CID_METERING_AREA_SET)]          = h8v_ctrl_set_metering_area,
    [V4L2_H8_CTRL_NR(V4L2_CID_ISO_MODE)]                   = h8v_ctrl_set_ISO,
    [V4L2_H8_CTRL_NR(V4L2_CID_SHARPNESS_MODE)]             = h8v_ctrl_set_sharpness,
    [V4L2_H8_CTRL_NR(V4L2_CID_ANTIBANDING_MODE)]           = h8v_ctrl_set_antibanding,

    [V4L2_H8_CTRL_NR(V4L2_CID_RGB2YUV_FORMAT_MODE)]        = h8v_ctrl_set_rgb2yuv_mode,
    [V4L2_H8_CTRL_NR(V4L2_CID_RGB2YUV_MATRIX_SET)]         = h8v_ctrl_set_rgb2yuv_matrix,
    [V4L2_H8_CTRL_NR(V4L2_CID_TONEMAP_MODE)]               = h8v_ctrl_set_tonemap,
    [V4L2_H8_CTRL_NR(V4L2_CID_TONEMAP_CURVE_SET)]          = h8v_ctrl_set_tonemap_curve,
    [V4L2_H8_CTRL_NR(V4L2_CID_SPECIAL_EFFECT_MODE)]        = h8v_ctrl_set_special_effect,

    [V4L2_H8_CTRL_NR(V4L2_CID_SCENARIO_MODE)]              = h8v_ctrl_set_scenario_mode,
    [V4L2_H8_CTRL_NR(V4L2_CID_SATURATION_MODE)]            = h8v_ctrl_set_saturation,
    [V4L2_H8_CTRL_NR(V4L2_CID_CONTRAST_MODE)]              = h8v_ctrl_set_contrast,
    [V4L2_H8_CTRL_NR(V4L2_CID_SCENE_MODE_SET)]             = h8v_ctrl_set_scene,
    [V4L2_H8_CTRL_NR(V4L2_CID_SHADINGCORRECT_MODE)]        = h8v_ctrl_set_shading_correct,

    [V4L2_H8_CTRL_NR(V4L2_CID_MANUAL_SHUTTER_SET)]         = h8v_ctrl_set_exposure_time,

    [V4L2_H8_CTRL_NR(V4L2_CID_MANUAL_WB_SET)]              = h8v_ctrl_set_manual_wb,

    [V4L2_H8_CTRL_NR(V4L2_CID_STREAM_INIT)]                = h8v_ctrl_set_stream_init,
    [V4L2_H8_CTRL_NR(V4L2_CID_MAIN_CTRL)]                  = h8v_ctrl_set_main,
};

int h8v_ctrl_set_main(struct h8_video_dev* dev)
{
    int i;
    h8v_ctrl_t func;
    TRACE_INFO(main_ctrl);

    // update stream new status
    if (dev->main_ctrl_data.stream_mask)
    {
        spin_lock(&dev->slock);
        for (i = INPUT_STREAM_ID_Preview; i < INPUT_STREAM_ID_MAX; i++)
        {
            if (dev->main_ctrl_data.stream_mask & (1<<i))
                dev->stream_init_data.stream_info[i] = dev->main_ctrl_data.stream_info[i];
        }
        spin_unlock(&dev->slock);
    }
    
    if (dev->main_ctrl_data.op_mask)
    {
        for (i = 0; i < V4L2_H8_CTRL_NR(V4L2_CID_STREAM_INIT); i++)
        {
            if (dev->main_ctrl_data.op_mask & (1<<i))
            {
                func = h8v_ctrls[i];
                func(dev);
            }
        }
    }

    return 0;
}
